__author__ = 'mnowotka'

from django.conf import settings
from django.db import DEFAULT_DB_ALIAS
from django.core.management.base import BaseCommand
from optparse import make_option

from blessings import Terminal
from progressbar import ProgressBar, RotatingMarker, Bar, Percentage, ETA
from chembl_core_model.models import ChemblIdLookup

from time import sleep
from random import random
from urlparse import urlparse
from threading import Thread
import httplib
from Queue import Queue

# ----------------------------------------------------------------------------------------------------------------------

term = Terminal()

WEB_UI_BASE_URL = getattr(settings, 'WEB_UI_BASE_URL', 'https://wwwdev.ebi.ac.uk')
TOTAL_RETRIES = getattr(settings, 'TOTAL_RETRIES', 3)
BACKOFF_FACTOR = getattr(settings, 'BACKOFF_FACTOR', 2)
CONCURRENT_SIZE = getattr(settings, 'CONCURRENT_SIZE', 5)
PROXIES = getattr(settings, 'PROXIES', None)
q = Queue(CONCURRENT_SIZE * 2)

WIDGETS = {
    'COMPOUND': [
        "/chembl/widget/create/compound_activity_chart/compound/activity/",
        "/chembl/widget/create/compound_activity_chart/compound/assay/",
        "/chembl/widget/create/compound_activity_chart/compound/target/",
    ],
    'TARGET': [
        "/chembl/widget/create/target_activity_chart/target/activity/",
        "/chembl/widget/create/target_assay_chart/target/assay/",
        "/chembl/widget/create/target_compoundmw_chart/target/compound_mw/",
        "/chembl/widget/create/target_compoundalogp_chart/target/compound_alogp/",
        "/chembl/widget/create/target_compoundpsa_chart/target/compound_psa/",
        "/chembl/widget/create/target_ligeff_chart/target/lig_eff/"
    ],
    'ASSAY': [
        "/chembl/widget/create/target_activity_chart/target/activity/",
        "/chembl/widget/create/target_assay_chart/target/assay/",
        "/chembl/widget/create/target_compoundmw_chart/target/compound_mw/",
        "/chembl/widget/create/target_compoundalogp_chart/target/compound_alogp/",
        "/chembl/widget/create/target_compoundpsa_chart/target/compound_psa/",
        "/chembl/widget/create/target_ligeff_chart/target/lig_eff/"
    ],
    'DOCUMENT': [
        "/chembl/widget/create/assay_activity_chart/assay/activity/",
        "/chembl/widget/create/assay_compoundmw_chart/assay/compound_mw/",
        "/chembl/widget/create/assay_compoundalogp_chart/assay/compound_alogp/",
        "/chembl/widget/create/assay_compoundpsa_chart/assay/compound_psa/",
        "/chembl/widget/create/assay_target_chart/assay/target/",
    ]}

# ----------------------------------------------------------------------------------------------------------------------

def doWork():
    while True:
        url = q.get()
        tries_left = TOTAL_RETRIES
        status, url = getStatus(url)
        tries_left -= 1
        while status != 200 and tries_left > 0:
            sleep(random())
            status, url = getStatus(url)
            if status != 429:
                tries_left -= 1
        if status != 200:
            print('url {0} returned status {1}'.format(url, status))
        q.task_done()

# ----------------------------------------------------------------------------------------------------------------------

def getStatus(ourl):
    try:
        url = urlparse(ourl)
        conn = httplib.HTTPConnection(url.netloc)
        conn.request("GET", url.path)
        res = conn.getresponse()
        return res.status, ourl
    except:
        return "error", ourl

# ----------------------------------------------------------------------------------------------------------------------


class Writer(object):
    """Create an object with a write method that writes to a
    specific place on the screen, defined at instantiation.
    This is the glue between blessings and progressbar.
    """
    def __init__(self, location):
        """
        Input: location - tuple of ints (x, y), the position
        of the bar in the terminal
        """
        self.location = location

    def write(self, string):
        with term.location(*self.location):
            print(string)

# ----------------------------------------------------------------------------------------------------------------------


class Command(BaseCommand):

    help = "Generate cache for ChEMBL web UI widgets."
    args = '[--database]'

# ----------------------------------------------------------------------------------------------------------------------

    def add_arguments(self, parser):
        parser.add_argument('--database', dest='db',
            default=DEFAULT_DB_ALIAS, help='database')

# ----------------------------------------------------------------------------------------------------------------------

    def handle(self, *args, **options):
        self.db = options.get('db')
        self.verbosity = options.get('verbosity')

        if self.verbosity > 1:
            print self.help
        with term.fullscreen():
            print(term.clear())
            for i in range(CONCURRENT_SIZE):
                t = Thread(target=doWork)
                t.daemon = True
                t.start()
            try:
                writer = Writer((0, 0))
                objects = ChemblIdLookup.objects.using(self.db).all()
                count = objects.count()
                pbar = ProgressBar(widgets=['OBJECTS: ', Percentage(), ' ', Bar(marker=RotatingMarker()), ' ', ETA()],
                    fd=writer, maxval=count).start()
                for i in range(0,count,CONCURRENT_SIZE):
                    chunk = objects[i:i+CONCURRENT_SIZE]
                    for obj in chunk:
                        for widget_url in WIDGETS[obj.entity_type]:
                            url = '{0}{1}{2}'.format(WEB_UI_BASE_URL, widget_url, obj.pk)
                            q.put(url)
                    q.join()
                    pbar.update(i)

                pbar.update(count)
                pbar.finish()

            except Exception as e:
                print e.message

# ----------------------------------------------------------------------------------------------------------------------







__author__ = 'mnowotka'

import django
from django.core.management.commands.inspectdb import Command as BaseCommand
from django.db import connections, DEFAULT_DB_ALIAS
from optparse import make_option
from django.core.management.base import CommandError
from progressbar import ProgressBar, RotatingMarker, Bar, Percentage, ETA, Counter
import sys
from colorama import init
from termcolor import colored
from django.conf import settings
from django.db import transaction

init()

LINE = '#' + '-' * 119

# ----------------------------------------------------------------------------------------------------------------------

INDEX_PREFIX_NAME = 'MOLIDX_CMPDMO%'

DEACTIVATE_OLD_CARTRIDGE_SQL = 'mdlaux.unsetup'

ACTIVATE_NEW_CARTRIDGE_SQL = 'C$DIRECT2016.MDLAUXOP.SETUP'

COLLECT_STATS = "DBMS_STATS.GATHER_TABLE_STATS"

DROP_INDEXES_SQL = """SELECT 'DROP ' || object_type || ' ' || object_name || ';'
FROM user_objects where object_name like %s;"""

DROP_TABLES_SQL = 'drop table COMPOUND_MOLS'

CREATE_TABLES_SQL = """
CREATE TABLE "COMPOUND_MOLS"
  (	"MOLREGNO" NUMBER(9,0),
	"CTAB" BLOB,
	 CONSTRAINT "PK_CMPDMOL_MOLREGNO" PRIMARY KEY ("MOLREGNO")
USING INDEX PCTFREE 10 INITRANS 2 MAXTRANS 255 COMPUTE STATISTICS
STORAGE(INITIAL 15728640 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1 BUFFER_POOL DEFAULT),
	 CONSTRAINT "FK_CMPDMOL_MOLREGNO" FOREIGN KEY ("MOLREGNO")
	  REFERENCES "MOLECULE_DICTIONARY" ("MOLREGNO") ENABLE
  ) PCTFREE 10 PCTUSED 40 INITRANS 1 MAXTRANS 255 NOCOMPRESS LOGGING
STORAGE(INITIAL 796917760 NEXT 1048576 MINEXTENTS 1 MAXEXTENTS 2147483645
PCTINCREASE 0 FREELISTS 1 FREELIST GROUPS 1 BUFFER_POOL DEFAULT)
LOB ("CTAB") STORE AS BASICFILE (
DISABLE STORAGE IN ROW )
        """

CREATE_INDEXES_SQL = """CREATE INDEX MOLIDX_CMPDMOL ON
        COMPOUND_MOLS(CTAB) INDEXTYPE IS "C$DIRECT2016"."MXIXMDL"
        PARAMETERS ('TEMPDIR=/transport/TEMPDIR');"""


COUNT_STRUCTURES_SQL = "SELECT COUNT(*) FROM compound_structures"

SELECT_MOLREGNO_SQL = "SELECT molregno FROM compound_structures"

ADD_COMPOUND_TO_INDEX_SQL = """insert into COMPOUND_MOLS (molregno, ctab)
select compound_structures.molregno, mol(compound_structures.molfile)
from compound_structures
where molregno=%s"""

COUNT_MOLECULES_SQL = "SELECT COUNT(*) FROM COMPOUND_MOLS"

# ----------------------------------------------------------------------------------------------------------------------


class Command(BaseCommand):
    help = "This script removes all cartridge related objects, activates new Biovia Direct cartridge and recreates " \
           "necessary objects and indexes."

    r_connection = None
    w_connection = None
    r_cursor = None
    w_cursor = None

# ----------------------------------------------------------------------------------------------------------------------

    def add_arguments(self, parser):
        parser.add_argument('--source', action='store', dest='source',
                            default=DEFAULT_DB_ALIAS, help='connection details for reading the database')
        parser.add_argument('--target', action='store', dest='target',
                            default=DEFAULT_DB_ALIAS + '_write', help='connection details for writing the database')

# -----------------------------------------------------------------------------------------------------------------------

    def handle(self, **options):
        if settings.DEBUG:
            print "Django is in debug mode, which causes memory leak. Set settings.DEBUG to False and run again."
            return

        django.db.reset_queries()
        self.handle_activation(options)

# ----------------------------------------------------------------------------------------------------------------------

    def handle_activation(self, options):

        # source and target should point to the same schema, we just need two separate DB connections
        # one for reading and one for writing
        source = options.get('source')
        target = options.get('target')

        with transaction.atomic(using=target):

            self.r_connection = connections[source]
            self.w_connection = connections[target]
            if self.r_connection.vendor != 'oracle' or self.w_connection.vendor != 'oracle':
                raise NotImplementedError
            self.r_cursor = self.r_connection.cursor()
            self.w_cursor = self.w_connection.cursor()

            self.try_execute_proc(DEACTIVATE_OLD_CARTRIDGE_SQL, [], 'Disabling old cartridge...', ['PLS-00201'])
            self.try_execute_proc(ACTIVATE_NEW_CARTRIDGE_SQL, [], 'Activating new cartridge...')

            print 'Dropping cartridge specific objects...'
            self.r_cursor.execute(DROP_INDEXES_SQL, [INDEX_PREFIX_NAME])
            result = self.r_cursor.fetchmany()
            while result:
                for drop_statement in result:
                    try:
                        self.w_cursor.execute(drop_statement[0])
                    except Exception as e:
                        if 'ORA-01418: specified index does not exist' in str(
                                e.message) or 'ORA-00942: table or view does not exist' in str(e.message) or 'ORA-02289: sequence does not exist' in str(e.message):
                            print (colored('Warning: %s' % e.message, 'yellow'))
                            continue
                        print (colored('Error dropping object using statement %s (%s)' % (drop_statement[0], e.message), 'red'))
                        raise e
                result = self.r_cursor.fetchmany()

        print (colored('Done.\n', 'green'))

        self.try_execute_sql(DROP_TABLES_SQL, [], 'Dropping molecules table..', ['ORA-00942'])
        self.try_execute_sql(CREATE_TABLES_SQL, [], 'Creating molecules table...')

        self.try_execute_sql(CREATE_INDEXES_SQL, [], 'Creating indexes... (This may take a while)')
        self.r_cursor.execute(COUNT_STRUCTURES_SQL)
        count = int(self.r_cursor.fetchall()[0][0])
        pbar = ProgressBar(widgets=['INDEXING OBJECTS: [', Counter(), '/%s' % count, '] ', Percentage(), ' ', Bar(marker=RotatingMarker()), ' ', ETA()],
                    maxval=count).start()
        self.r_cursor.execute(SELECT_MOLREGNO_SQL)
        result = self.r_cursor.fetchmany()
        counter = 0
        failed = 0
        with self.w_connection.constraint_checks_disabled():
                while result:
                    with transaction.atomic(using=target):
                        for molregno in result:
                            try:
                                self.w_cursor.execute(ADD_COMPOUND_TO_INDEX_SQL, molregno)
                            except Exception as e:
                                failed += 1
                                print (colored('Error generating binary object for molecule %s (%s)' % (molregno, e.message), 'red'))
                            counter += 1
                            pbar.update(counter)
                        result = self.r_cursor.fetchmany(1000)

        pbar.update(count)
        pbar.finish()

        self.r_cursor.execute(COUNT_MOLECULES_SQL)
        total = int(self.r_cursor.fetchall()[0][0])

        print 'Created %s records in compound_mols.' % total
        print 'There was %s records in compounds structures' % count
        print 'Out of which %s records caused errors' % failed
        print "count - failed = %s, total = %s" % (count-failed, total)

        print 'Dropping cartridge specific objects...'
        self.r_cursor.execute(DROP_INDEXES_SQL, [INDEX_PREFIX_NAME])
        result = self.r_cursor.fetchmany()
        while result:
            for drop_statement in result:
                try:
                    self.w_cursor.execute(drop_statement[0])
                except Exception as e:
                    if 'ORA-01418: specified index does not exist' in str(
                            e.message) or 'ORA-00942: table or view does not exist' in str(e.message) or 'ORA-02289: sequence does not exist' in str(e.message):
                        print (colored('Warning: %s' % e.message, 'yellow'))
                        continue
                    print (colored('Error dropping object using statement %s (%s)' % (drop_statement[0], e.message), 'red'))
                    raise e
            result = self.r_cursor.fetchmany()
        transaction.commit(using=target)
        print (colored('Done.\n', 'green'))

        self.try_execute_sql(CREATE_INDEXES_SQL, [], 'Creating indexes... (This may take a while)')

        self.try_execute_proc(COLLECT_STATS, ['CHEMBL_APP', 'COMPOUND_MOLS'], 'Collecting stats..', [])

        self.r_connection.close()
        self.w_connection.close()

# ----------------------------------------------------------------------------------------------------------------------

    def try_execute_sql(self, sql, params, desc, ignores=None):
        print LINE + '\n'
        print desc
        if not ignores:
            ignores = []
        try:
            self.r_cursor.execute(sql, params)
        except Exception as e:
            if any(ign in str(e.message) for ign in ignores):
                print (colored('Warning: %s' % e.message, 'yellow'))
                return
            print (colored('Failed: %s' % e.message, 'red'))
            sys.exit()
        print (colored('Done.\n', 'green'))

# ----------------------------------------------------------------------------------------------------------------------

    def try_execute_proc(self, proc, params, desc, ignores=None):
        print LINE + '\n'
        print desc
        if not ignores:
            ignores = []
        try:
            self.r_cursor.callproc(proc, params)
        except Exception as e:
            if any(ign in str(e.message) for ign in ignores):
                print (colored('Warning: %s' % e.message, 'yellow'))
                return
            print (colored('Failed: %s' % e.message, 'red'))
            sys.exit()
        print (colored('Done.\n', 'green'))

# ----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

import django
from django.core.management.commands.inspectdb import Command as BaseCommand
from django.db import connections
from optparse import make_option
from progressbar import ProgressBar, RotatingMarker, Bar, Percentage, ETA
from django.conf import settings
from colorama import init
from termcolor import colored
import sys, os
import gzip, bz2

init()


try:
    from chembl_compatibility.models import TargetDictionary
except ImportError:
    from chembl_core_model.models import TargetDictionary


# ----------------------------------------------------------------------------------------------------------------------


class Command(BaseCommand):
    help = "This script generates a fasta file of chembl data"

# ----------------------------------------------------------------------------------------------------------------------

    def add_arguments(self, parser):
        parser.add_argument('--filename', action='store', dest='filename', default=None, help='Output file')

# ----------------------------------------------------------------------------------------------------------------------

    def handle(self, **options):
        if settings.DEBUG:
            print "Django is in debug mode, which causes memory leak. Set settings.DEBUG to False and run again."
            return

        django.db.reset_queries()
        qs = TargetDictionary.objects.all().values_list('tid', 'pref_name', 'organism',
                                                        'targetcomponents__component__sequence', 'description',
                                                        'chembl_id',
                                                        'targetcomponents__component__accession').order_by('tid')
        count = qs.count()

        filename = options.get('filename')
        if not filename:
            file = sys.stdout
        else:
            _, file_extension = os.path.splitext(filename)
            if file_extension == '.txt':
                file = open(filename, "w") if filename else sys.stdout
            elif file_extension == '.gz':
                file = gzip.open(filename, 'wb')
            elif file_extension == '.bz2':
                file = bz2.BZ2File(filename, 'w')
            else:
                print (colored('Unrecognized file extension: %s' % file_extension, 'red'))
                sys.exit()

        pbar = ProgressBar(
            widgets=['CREATING BLAST DB: ', Percentage(), ' ', Bar(marker=RotatingMarker()), ' ', ETA()],
            maxval=count,
            fd=open(os.devnull, "w") if not filename else sys.stderr).start()

        counter = 0

        for res in qs:
            if res[3]:
                name = res[1] or res[4]
                accession = '[%s]' % res[6] if res[6] else ''
                file.write(">%s %s %s (%s)\n%s\n" % (res[5], accession, name, res[2], res[3]))
            pbar.update(counter)
            counter += 1
        pbar.update(count)
        pbar.finish()

# ----------------------------------------------------------------------------------------------------------------------



__author__ = 'mnowotka'

import django
from django.core.management.commands.inspectdb import Command as BaseCommand
from django.db import connections
from optparse import make_option
from progressbar import ProgressBar, RotatingMarker, Bar, Percentage, ETA
from django.conf import settings
import sys, os
import gzip, bz2

try:
    from chembl_compatibility.models import TargetDictionary
except ImportError:
    from chembl_core_model.models import TargetDictionary

try:
    from chembl_compatibility.models import MoleculeDictionary
except ImportError:
    from chembl_core_model.models import MoleculeDictionary

SETTINGS = {
    'DOWNLOAD': [
        {
            'filename': 'chembl_22.fa.gz',
            'values': ['chembl_id', 'targetcomponents__component__accession',
                       'targetcomponents__component__description', 'targetcomponents__component__organism',
                       'targetcomponents__component__sequence'],
            'model': TargetDictionary,
            'filters': {'targetcomponents__component__component_type': 'PROTEIN'},
            'line_template': '> %(chembl_id)s [%(targetcomponents__component__accession)s] %(targetcomponents__component__description)s (%(targetcomponents__component__organism)s)\n%(targetcomponents__component__sequence)s\n',
        },
        {
            'filename': 'chembl_22_bio.fa.gz',
            'values': ['chembl_id', 'biotherapeutics__bio_component_sequences__component_id',
                       'biotherapeutics__bio_component_sequences__description',
                       'biotherapeutics__bio_component_sequences__organism',
                       'biotherapeutics__bio_component_sequences__sequence'],
            'model': MoleculeDictionary,
            'filters': {'biotherapeutics__bio_component_sequences__sequence__isnull': False},
            'line_template': '> %(chembl_id)s [%(biotherapeutics__bio_component_sequences__component_id)s] %(biotherapeutics__bio_component_sequences__description)s (%(biotherapeutics__bio_component_sequences__organism)s)\n%(biotherapeutics__bio_component_sequences__sequence)s\n',
        }
    ],
    'INTERFACE': [
        {
            'filename': 'chembl_22.fa',
            'values': ['tid', 'pref_name', 'organism', 'targetcomponents__component__sequence'],
            'model': TargetDictionary,
            'filters': {'targetcomponents__component__component_type': 'PROTEIN'},
            'line_template': '>%(tid)s %(pref_name)s (%(organism)s)\n%(targetcomponents__component__sequence)s\n',
        },
        {
            'filename': 'chembl_22_mab.fa',
            'values': ['biotherapeutics__bio_component_sequences__component_id',
                       'biotherapeutics__bio_component_sequences__sequence'],
            'model': MoleculeDictionary,
            'filters': {'biotherapeutics__bio_component_sequences__sequence__isnull': False},
            'line_template': '> %(biotherapeutics__bio_component_sequences__component_id)s\n%(biotherapeutics__bio_component_sequences__sequence)s\n',
        }
    ]
}

# ----------------------------------------------------------------------------------------------------------------------


class Command(BaseCommand):

    help = "This script generates a blast db out of chembl data"

# ----------------------------------------------------------------------------------------------------------------------

    def add_arguments(self, parser):
        parser.add_argument('--model', action='store', dest='mode', help='mode (can be "download" or "interface")')

# ----------------------------------------------------------------------------------------------------------------------

    def handle(self, **options):
        if settings.DEBUG:
            print "Django is in debug mode, which causes memory leak. Set settings.DEBUG to False and run again."
            return

        mode = options.get('mode')

        files = SETTINGS[mode.upper()]
        for file in files:
            self.db_to_file(**file)

# ----------------------------------------------------------------------------------------------------------------------

    def db_to_file(self, values, filename, model, filters, line_template):
        django.db.reset_queries()
        qs = model.objects.all()
        if filters:
            qs = qs.filter(**filters)
        qs = qs.values(*values)
        count = qs.count()

        if not filename:
            file = sys.stdout
        else:
            _, file_extension = os.path.splitext(filename)
            if file_extension == '.gz':
                file = gzip.open(filename, 'wb')
            elif file_extension == '.bz2':
                file = bz2.BZ2File(filename, 'w')
            else:
                file = open(filename, "w") if filename else sys.stdout

        pbar = ProgressBar(
            widgets=['Writing %s: ' % filename, Percentage(), ' ', Bar(marker=RotatingMarker()), ' ', ETA()],
            maxval=count,
            fd=open(os.devnull, "w") if not filename else sys.stderr).start()

        counter = 0

        for res in qs:
            line = line_template % res
            line = line.replace(' (None)', '')
            line = line.replace(' None', '')
            line = line.replace(' \n', '\n')
            file.write(line)
            pbar.update(counter)
            counter += 1
        pbar.update(count)
        pbar.finish()

# ----------------------------------------------------------------------------------------------------------------------




__author__ = 'mnowotka'

import django
from django.core.management import call_command
from django.core.management.commands.inspectdb import Command as BaseCommand
from django.template.loader import render_to_string
from django.db import connections
from optparse import make_option
from progressbar import ProgressBar, RotatingMarker, Bar, Percentage, ETA
from django.conf import settings
import tempfile
import shutil
import sys, os
from datetime import date
from django.db import connections
import hashlib
import subprocess
from dateutil.parser import parse
import tarfile
import gzip

try:
    from chembl_compatibility.models import TargetDictionary
except ImportError:
    from chembl_core_model.models import TargetDictionary

try:
    from chembl_compatibility.models import MoleculeDictionary
except ImportError:
    from chembl_core_model.models import MoleculeDictionary


DUMP_COMMANDS = {
    'postgres': 'pg_dump -h %(HOST)s -p %(PORT)s -U %(USER)s %(NAME)s -O --no-tablespaces -f %(FILENAME)s -W',
    'mysql': 'mysqldump -u %(USER)s -h %(HOST)s --protocol=tcp -P %(PORT)s --default-character-set=utf8 -p%(PASSWORD)s --skip-triggers %(NAME)s -r %(FILENAME)s',
    'sqlite': 'sqlite3 %(NAME)s < make_chemreps',
    'oracle': 'exp %(USER)s/%(PASSWORD)s@%(NAME)s file=%(FILENAME)s OWNER=%(USER)s GRANTS=N STATISTICS=NONE'
}

ORACLE_DUMPS = {
    '9i': {
        "ORACLE_BASE": "/sw/arch/dbtools/oracle",
        "ORACLE_HOME": "/product/9.2.0.7",
        "TNS_ADMIN": "/network/admin",
        "ORA_NLS32": "/ocommon/nls/admin/data",
        "NLS_LANG": "AMERICAN_AMERICA.US7ASCII",
        "NLS_DATE_FORMAT": "DD-MON-RRRR HH24:MI:SS",
    },

    '10g': {
        "ORACLE_BASE": "/sw/arch/x86_64-linux/dbtools/oracle",
        "ORACLE_HOME": "/product/10.2.0_msd",
        "TNS_ADMIN": "/network/admin",
        "ORA_NLS32": "/nls/data",
        "NLS_LANG": "AMERICAN_AMERICA.al32utf8",
        "NLS_DATE_FORMAT": "DD-MON-RRRR HH24:MI:SS",
    },

    '11g': {
        "ORACLE_BASE": "/sw/arch/x86_64-linux/dbtools/oracle",
        "ORACLE_HOME": "/product/11.1.0.6.2/client",
        "TNS_ADMIN": "/network/admin",
        "ORA_NLS32": "/nls/data",
        "NLS_LANG": "AMERICAN_AMERICA.al32utf8",
        "NLS_DATE_FORMAT": "DD-MON-RRRR HH24:MI:SS",
    }
}

DB_VERSIONS = {
    'mysql_ver': '5.0',
    'postgres_ver': '9.1.4',
    'sqlite_ver': '3.8.5',
}

# ----------------------------------------------------------------------------------------------------------------------


def make_env(version):
    env = os.environ.copy()
    ret = ORACLE_DUMPS.get(version, {}).copy()
    ret["ORACLE_HOME"] = ret["ORACLE_BASE"] + ret["ORACLE_HOME"]
    ret["TNS_ADMIN"] = ret["ORACLE_HOME"] + ret["TNS_ADMIN"]
    ret["ORA_NLS32"] = ret["ORACLE_HOME"] + ret["ORA_NLS32"]
    ret["LD_LIBRARY_PATH"] = ret["ORACLE_HOME"] + '/lib:' + env["LD_LIBRARY_PATH"]
    ret["PATH"] = ret["ORACLE_HOME"] + '/bin:' + env["PATH"]
    return ret

# ----------------------------------------------------------------------------------------------------------------------


def make_tarfile(output_filename, source_dir):
    with tarfile.open(output_filename, "w:gz") as tar:
        tar.add(source_dir, arcname=os.path.basename(source_dir))
    shutil.rmtree(source_dir)

# ----------------------------------------------------------------------------------------------------------------------


def sha256_for_file(path, block_size=4096):
    try:
        with open(path, 'rb') as rf:
            h = hashlib.sha256()
            for chunk in iter(lambda: rf.read(block_size), b''):
                h.update(chunk)
        return h.hexdigest()
    except IOError:
        return None, path

# ----------------------------------------------------------------------------------------------------------------------


def make_checksums_file(directory, file='checksums.txt'):
    with open(os.path.join(directory, file), 'w') as f:
        f.write('file\tSHA256\n\n')
        for _, _, filenames in os.walk(directory, topdown=False):
            for filename in filenames:
                hash = sha256_for_file(os.path.join(directory, filename))
                f.write('%s\t%s\n' % (filename, hash))

# ----------------------------------------------------------------------------------------------------------------------


def gzip_file(file_path):
    f_in = open(file_path)
    f_out = gzip.open(file_path + '.gz', 'wb')
    f_out.writelines(f_in)
    f_out.close()
    f_in.close()
    os.remove(file_path)

# ----------------------------------------------------------------------------------------------------------------------


class cd:
    """Context manager for changing the current working directory"""
    def __init__(self, newPath):
        self.newPath = os.path.expanduser(newPath)

    def __enter__(self):
        self.savedPath = os.getcwd()
        os.chdir(self.newPath)

    def __exit__(self, etype, value, traceback):
        os.chdir(self.savedPath)

# ----------------------------------------------------------------------------------------------------------------------


class Command(BaseCommand):

    help = "This script generates a blast db out of chembl data"

# ----------------------------------------------------------------------------------------------------------------------

    def add_arguments(self, parser):
        parser.add_argument('--release', action='store', dest='release', help='release name, for example chembl_14')
        parser.add_argument('--date', action='store', dest='date', help='release date, default is today')
        parser.add_argument('--contact', action='store', dest='contact', default="chembl-help@ebi.ac.uk",
                            help='contact email, default is chembl-help@ebi.ac.uk')

# ----------------------------------------------------------------------------------------------------------------------

    def handle(self, **options):
        if settings.DEBUG:
            print "Django is in debug mode, which causes memory leak. Set settings.DEBUG to False and run again."
            return

        self.release = options.get('release')
        self.date = parse(options.get('date') or '')
        self.contact = options.get('contact')

        if not self.release:
            print 'No release name given, please specify release name (for example chembl_14)'
            return

        base_folder = tempfile.mkdtemp()

        print 'Saving to %s' % base_folder

        new_location = os.path.join(base_folder, self.release)
        os.mkdir(new_location)

        mysql_dir = os.path.join(new_location,'%s_mysql' % self.release)
        postgres_dir = os.path.join(new_location,'%s_postgresql' % self.release)
        sqlite_dir = os.path.join(new_location,'%s_sqlite' % self.release)
        oracle_11g_dir = os.path.join(new_location,'%s_oracle11g' % self.release)
        oracle_10g_dir = os.path.join(new_location,'%s_oracle10g' % self.release)
        oracle_9i_dir = os.path.join(new_location,'%s_oracle9i' % self.release)

        os.mkdir(mysql_dir)
        os.mkdir(postgres_dir)
        os.mkdir(sqlite_dir)
        os.mkdir(oracle_11g_dir)
        os.mkdir(oracle_10g_dir)
        os.mkdir(oracle_9i_dir)

        self.template_2_file(os.path.join(new_location, 'LICENSE'), 'chembl_extras/LICENSE.tmpl')
        self.template_2_file(os.path.join(new_location, 'README'), 'chembl_extras/README.tmpl')
        self.template_2_file(os.path.join(new_location, 'REQUIRED.ATTRIBUTION'), 'chembl_extras/REQUIRED.ATTRIBUTION.tmpl')

        self.template_2_file(os.path.join(mysql_dir, 'INSTALL'), 'chembl_extras/mysql/INSTALL.tmpl')
        self.template_2_file(os.path.join(oracle_11g_dir, 'INSTALL'), 'chembl_extras/oracle_11g/INSTALL.tmpl')
        self.template_2_file(os.path.join(oracle_10g_dir, 'INSTALL'), 'chembl_extras/oracle_10g/INSTALL.tmpl')
        self.template_2_file(os.path.join(oracle_9i_dir, 'INSTALL'), 'chembl_extras/oracle_9i/INSTALL.tmpl')
        self.template_2_file(os.path.join(postgres_dir, 'INSTALL'), 'chembl_extras/postgres/INSTALL.tmpl')
        self.template_2_file(os.path.join(sqlite_dir, 'INSTALL'), 'chembl_extras/sqlite/INSTALL.tmpl')

        print 'OK, directory structure, installation instruction, licenses and readme files are now in place.'
        print 'Generating other files...'

        with cd(new_location):
            call_command('create_blastdb', mode="download")
            # call_command('sdfexport')

        # self.make_dump('oracle', os.path.join(oracle_9i_dir, '%s_9i.dmp' % self.release), my_env=make_env('9i'))
        # self.make_dump('oracle', os.path.join(oracle_9i_dir, '%s_10g.dmp' % self.release), my_env=make_env('10g'))
        # self.make_dump('oracle', os.path.join(oracle_9i_dir, '%s_11g.dmp' % self.release), my_env=make_env('11g'))
        # self.make_dump('postgres', os.path.join(postgres_dir, '%s.pgdump.sql' % self.release))
        # self.make_dump('mysql', os.path.join(mysql_dir, '%s.mysqldump.sql' % self.release))
        # self.template_2_file('make_chemreps', 'chembl_extras/sqlite/make_chemreps.tmpl')
        # chemreps = os.path.join(new_location, '%s_chemreps.txt' % self.release)
        # self.make_dump('sqlite', chemreps)
        # gzip_file(chemreps)
        # os.remove('make_chemreps')

        print 'compressing data dumps...'
        make_tarfile(os.path.join(new_location,'%s_mysql.tar.gz' % self.release), mysql_dir)
        make_tarfile(os.path.join(new_location,'%s_postgresql.tar.gz' % self.release), postgres_dir)
        make_tarfile(os.path.join(new_location,'%s_sqlite.tar.gz' % self.release), sqlite_dir)
        make_tarfile(os.path.join(new_location,'%s_oracle11g.tar.gz' % self.release), oracle_11g_dir)
        make_tarfile(os.path.join(new_location,'%s_oracle10g.tar.gz' % self.release), oracle_10g_dir)
        make_tarfile(os.path.join(new_location,'%s_oracle9i.tar.gz' % self.release), oracle_9i_dir)

        make_checksums_file(new_location)

        # shutil.rmtree(base_folder)


# ----------------------------------------------------------------------------------------------------------------------

    def template_2_file(self, filepath, template_name):
        context = {'release_version': self.release, 'release_date': self.date, 'contact': self.contact}
        context.update(DB_VERSIONS)
        content = render_to_string(template_name, context)
        with open(filepath, 'w') as f:
            f.write(content)

# ----------------------------------------------------------------------------------------------------------------------

    def make_dump(self, engine, output, my_env=None):

        if engine not in connections.databases:
            print 'There is no connection named %s in the django settings so, dumping %s will be ignored' % (engine, engine)
            return
        if engine not in DUMP_COMMANDS:
            print "I don't know the dump command for the %s engine. Ignoring." % engine
            return
        print 'Generating %s data dump' % engine
        conn = connections.databases[engine].copy()
        conn.update({'FILENAME': output})
        command = 'time(%s)' % (DUMP_COMMANDS[engine] % conn)
        print 'command:'
        print command
        env = None
        if my_env:
            env = os.environ.copy()
            env.update(my_env)
        proc = subprocess.Popen(command, stdin=sys.stdin, stdout=sys.stdout, stderr=sys.stderr, shell=True, env=env)
        proc.communicate()
        print '%s dump process has finished with the status code %s' % (engine, proc.returncode)

# ----------------------------------------------------------------------------------------------------------------------






__author__ = 'mnowotka'

from chembl_core_model.models import CompoundStructures
from django.core.management.base import BaseCommand
from optparse import make_option
from clint.textui import progress
from django.conf import settings
from django.db import DEFAULT_DB_ALIAS
import gzip

# ----------------------------------------------------------------------------------------------------------------------


class Command(BaseCommand):

    help = "Export all molecules from ChEMBL to one compressed *.sdf file."
    args = '[--chebi_id, --chembl_id, --downgraded, --out]'

# ----------------------------------------------------------------------------------------------------------------------

    def add_arguments(self, parser):
        parser.add_argument('--chebi_id', default=False, dest='chebi_id', help='Include chebi_id.')
        parser.add_argument('--chembl_id', default=True, dest='chembl_id', help='Include chembl_id.')
        parser.add_argument('--downgraded', default=False, dest='downgraded', help='Include downgraded compounds.')
        parser.add_argument('--out', dest='out_file', default='chembl.sdf.gz', help='Output file')
        parser.add_argument('--database', dest='db', default=DEFAULT_DB_ALIAS, help='database')

# ----------------------------------------------------------------------------------------------------------------------

    def handle(self, *args, **options):

        if settings.DEBUG:
            print "Django is in debug mode, which causes memory leak. Set settings.DEBUG to False and run again."
            return

        filename = options.get('out_file')
        chebi_id = options.get('chebi_id')
        chembl_id = options.get('chembl_id')
        downgraded = options.get('downgraded')
        db = options.get('db')
        verbosity = options.get('verbosity')

        if verbosity > 1:
            print self.help

        filters = {'molecule__chembl__entity_type':'COMPOUND'}

        if not downgraded:
            filters['molecule__downgraded'] = False

        fields = ['molfile']
        mol_template = '{molfile}\n'

        fields.append('molecule__chembl__chembl_id')
        if chembl_id:
            mol_template += '> <chembl_id>\n{molecule__chembl__chembl_id}\n\n'

        if chebi_id:
            fields.append('molecule__chebi_id')
            mol_template += '> <chebi_id>\n{molecule__chebi_id}\n\n'

        mol_template += '$$$$\n'

        structures = CompoundStructures.objects.using(db).filter(**filters)
        n_structures = structures.count()

        if verbosity > 1:
            print "Found {0} structures to export".format(n_structures)

        if not n_structures:
            print "Nothing to export..."

        else:
            f = gzip.open(filename, 'wb')
            try:
                for raw in progress.bar(structures.values(*fields).iterator(),
                            label="exporting... ", expected_size=n_structures):
                    if raw['molfile'] and isinstance(raw['molfile'], basestring):
                        raw['molfile'] = raw['molfile'].rstrip()
                    mol = raw['molecule__chembl__chembl_id'] + mol_template.format(**raw)
                    f.write(mol)

            finally:
                f.close()

        if verbosity > 1:
            print "Exporting done."

# ----------------------------------------------------------------------------------------------------------------------
